#  ==========================================================================
#  loader.rb
#  --------------------------------------------------------------------------
#
#  Custom Toolbars Loader
#
#  by Jim Foltz, <jim.foltz@gmail.com>
#
#  --------------------------------------------------------------------------
#  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
#  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#  --------------------------------------------------------------------------
#  Changes:
#
#  0.2.0 : 2008-04-28 : beta release candidate
#
#  0.2.1 : 2008-05-18 : beta release candidate - rev 1
#
#  1.0.0 : 2008-06-01 : initial public release : by Jim Foltz
#
#  1.1.0 : 2010-09-08 : update : by Jim Foltz
#
#   ~ Updated toolbar restore with delay timer 
#
#  1.2.0 : 2011-02-15 : update : by Dan Rathbun
#
#   ~ Avoids SetNorthAngle tool BugSplat! in Sketchup v 8.x (as of M1)
#     |
#     * updated: Plugins/Toolbars/Files/Sketchup/tvp.rbs
#
#   ~ Fixed restore last state; on PC did not work correctly if never shown.
#     We now show then hide toolbars the 1st time loaded, so they will later
#     be shown floating. This prevents the "ol' toolbar shuffle."
#     |
#     * updated: Plugins/Toolbars/Files/loader.rb
#
#   ~ Provide new extension registration script with version number.
#     |
#     * update: Plugins/toolbars_v1_2_0.rb
#
#  ==========================================================================

#Sketchup.send_action "showRubyPanel:"
#
MAC =( Object::RUBY_PLATFORM =~ /darwin/i ? true : false ) unless defined?(MAC)
PC  =( not MAC ) unless defined?(PC)

module JF

  module CustomToolbars

    def ro
      Sketchup.active_model.rendering_options
    end

    def self.tool_validation_proc(n)
      if Sketchup.active_model.tools.active_tool_id != 0
        if Sketchup.active_model.tools.active_tool_name == n
          MF_CHECKED 
        else
          MF_UNCHECKED
        end
      end
    end

    def self.get_image_basename(cmd)
      su_ver  = Sketchup.version.to_i
      cwd     = File.dirname(__FILE__)
      search_paths = []

      if su_ver >=  14
        search_paths << File.join(cwd, 'sketchup', 'images', "20#{su_ver}")
      end

      contrib = File.join(cwd, 'sketchup', 'images', 'contrib')
      original = File.join(cwd, 'sketchup', 'images')
      search_paths << contrib
      search_paths << original

      cmd_basename = File.basename(cmd, ".rb")

      search_paths.each do |img_dir|
        img_path = File.join(img_dir, cmd_basename)
        if File.exist?(img_path+"_lg.png")
          return img_path
        end
      end
      return File.join(contrib, "_no_icon")
    end

    dir = File.dirname(__FILE__)

    scmddir = File.join(dir, "sketchup", "commands")
    tb_dir  = dir + "/.."
    tbs     = Dir[tb_dir + "/*.txt"]
    imgdir  = File.join(dir, "sketchup", "images")
    @error_list = []

    tbs.each do |tb|
      name = File.basename(tb, ".txt")
      ttb = UI::Toolbar.new("~ " + name)

      IO.foreach(tb) do |cmd|
        @cmd = nil
        cmd.strip!
        next if cmd[0, 1] == "#"
        cmd, cmt =  cmd.split("#")
        next if cmd.nil?
        cmd.strip!
        if cmd == "---"
          ttb.add_separator
          next
        end
        rb = File.join(scmddir, cmd + ".rb")
        if File.exists? rb
          puts "CT loading: #{rb}" if $JFDEBUG
          begin
            JF::CustomToolbars.module_eval(File.read(rb))
          rescue LoadError=> e
            @error_list << e.message << e.backtrace.join("\n")
          end
          #load rb
          if @cmd
            # set toolbar images
            @cmd.large_icon = get_image_basename(rb) + "_lg.png"
            @cmd.small_icon = get_image_basename(rb) + "_sm.png"
          end
          ttb.add_item(@cmd) if(@cmd)
        end
        @cmd = nil
      end

      # show ttb
      #
      state = ttb.get_last_state 
      if (state == TB_VISIBLE)
        ttb.restore
        ### v 1.1.0
        # Per bug 2902434; resize regression on PC
        #
        UI.start_timer(0.1, false) { ttb.restore } if PC
        #
        ###
        ### elseif state == TB_NEVER_SHOWN :
        # v 1.0.0 was: ttb.show
        # v 1.1.0 elseif was removed
        # v 1.2.0 Added back to show, then make state TB_HIDDEN
        #
      elsif (state == TB_NEVER_SHOWN)
        # We DONT want to show them until the user wants them,
        # but we cannot leave the state as TB_NEVER_SHOWN, as
        # there is a PC bug that inserts phantom empty toolbar
        # windows up near the menubar, if state is -1.
        # ALSO, IF we do not show them when first loaded, they
        # will get shown on the top toolbar container later,
        # leading to the "ol' toolbar shuffle."
        # We show, then hide toolbars using a start_timer, so
        # that the next time SU starts, they will float.
        ttb.show;  UI.start_timer(0.3, false) { ttb.hide }
        #
        ###
      end

    end # tbs.each do 

    def self.show_load_errors
      UI.messagebox(@error_list.join("\n\n"), MB_MULTILINE, "Custom Toolbars Load Errors")
    end

    def self.command_list_dialog
      @dlg = UI::WebDialog.new("All Commands")
      script  = '<script>'
      script << 'function cmd(cmd_name) {'
      script << '  window.location="skp:cmd@" + cmd_name;'
      script << '}'
      script << '</script>'
      style  = '<style>'
      style << '* { font: menu; }'
      style << 'div {cursor: pointer;}'
      style << 'img {margin-right: 11px; display: inline-block; width: 17px;}'
      style << '</style>'
      html = "<doctype html><html><head>#{style}#{script}</head><body>"
      cwd = File.dirname(__FILE__)
      cmd_dir = File.join(cwd, 'sketchup', 'commands')
      img_dir = File.join(cwd, 'sketchup', 'images')
      cmd_s = "cmd"
      Dir["#{cmd_dir}/*"].each do |cmd|
        cmd_name = File.basename(cmd, ".rb")
        html << "<div onclick=\"cmd('#{cmd_name}');\">"
        html << "<img src=\"#{get_image_basename(cmd_name)}_sm.png\"/>"
        html << "#{cmd_name}"
        html << "</div>\n"
      end
      html << "</body></html>"
      @dlg.add_action_callback('cmd') {|d, a|
        #puts "cmd:#{a.inspect}"
      }
      @dlg.set_html(html)
      @dlg.show
    end

    menu = UI.menu("Plugins").add_submenu("JF Custom Toolbars")
    menu.add_item("All Commands") { command_list_dialog }
    menu.add_item("Example Toolbar") {
      file = File.join(dir, "..", "Edge Styles Example.txt")
      if PC
        file.tr!('/', '\\\\')
        UI.openURL( "#{file}" )
      else
        UI.openURL( "file://#{file}" )
      end
    }
    id = menu.add_item("Show Errors") { show_load_errors }
    #menu.add_item("Browse Toolbars") { UI.openURL(dir) }
    #menu.add_item("About") { UI.openURL(dir) }


  end # module CustomToolbars
end # module JF
